#include "std/errno.h"
#include "thread/Scheduler.h"
#include <sys/types.h>

#define ID_PID 0
#define ID_PPID 1
#define ID_UID 2
#define ID_EUID 3
#define ID_GID 4
#define ID_EGID 5

void sys_getprocid(Context* context, int field)
{
    if (field == ID_PID)
    {
        context->rax = Scheduler::current_task()->id;
        return;
    }
    else if (field == ID_PPID)
    {
        context->rax = Scheduler::current_task()->ppid;
        return;
    }
    else if (field == ID_UID)
    {
        context->rax = Scheduler::current_task()->uid;
        return;
    }
    else if (field == ID_EUID)
    {
        context->rax = Scheduler::current_task()->euid;
        return;
    }
    else if (field == ID_GID)
    {
        context->rax = Scheduler::current_task()->gid;
        return;
    }
    else if (field == ID_EGID)
    {
        context->rax = Scheduler::current_task()->egid;
        return;
    }
    else
    {
        context->rax = -EINVAL;
        return;
    }
}

void sys_setuid(Context* context, uid_t uid)
{
    Task* current_task = Scheduler::current_task();

    if (!current_task->is_superuser())
    {
        if (uid != current_task->uid && uid != current_task->euid)
        {
            context->rax = -EPERM;
            return;
        }
    }

    current_task->uid = uid;
    current_task->euid = uid;

    context->rax = 0;
}

void sys_seteuid(Context* context, uid_t euid)
{
    Task* current_task = Scheduler::current_task();

    if (!current_task->is_superuser())
    {
        if (euid != current_task->uid)
        {
            context->rax = -EPERM;
            return;
        }
    }

    current_task->euid = euid;

    context->rax = 0;
}

void sys_setgid(Context* context, gid_t gid)
{
    Task* current_task = Scheduler::current_task();

    if (!current_task->is_superuser())
    {
        if (gid != current_task->gid && gid != current_task->egid)
        {
            context->rax = -EPERM;
            return;
        }
    }

    current_task->gid = gid;
    current_task->egid = gid;

    context->rax = 0;
}

void sys_setegid(Context* context, gid_t egid)
{
    Task* current_task = Scheduler::current_task();

    if (!current_task->is_superuser())
    {
        if (egid != current_task->gid)
        {
            context->rax = -EPERM;
            return;
        }
    }

    current_task->egid = egid;

    context->rax = 0;
}